require 'spec_helper'
require 'storing/postgres/adaptor'

describe Storing::Postgres::Adaptor do
	let(:adaptor) { Storing::Postgres::Adaptor.new(DB) }

	before do
		DB[:person].insert(:first_name => 'Peter', :last_name => 'Parker', :age => 23)
		DB[:person].insert(:first_name => 'Tony', :last_name => 'Stark', :age => 37)
		DB[:person].insert(:first_name => 'Bruce', :last_name => 'Banner', :age => 33)
	end

	describe "#select" do
		it "returns all records" do
			results = adaptor.select(:person)
			results.map{|p| p[:first_name]}.must_equal ["Peter", "Tony", "Bruce"]
		end

		it "returns records conditionally" do
			results = adaptor.select(:person, :first_name => 'Peter')
			results.length.must_equal 1
		end
	end

	describe "#insert" do
		it "inserts record" do
			adaptor.insert(:person, :first_name => 'Charles', :last_name => 'Xavier', :age => 67)
			DB[:person][:first_name => 'Charles'].wont_be_nil
		end

		it "returns primary key" do
			pk = adaptor.insert(:person, :first_name => 'Charles', :last_name => 'Xavier', :age => 67)
			pk.to_s.must_match /^\d+$/
		end	
	end

	describe "#update" do
		it "updates all records" do
			adaptor.update(:person, :first_name => 'Jim')
			DB[:person].each {|person| person[:first_name].must_equal 'Jim' }
		end

		it "updates records conditionally" do
			adaptor.update(:person, {:first_name => "Pete"}, {:last_name => 'Parker'})
			DB[:person].map {|person| person[:first_name] }.must_equal ["Tony", "Bruce", "Pete"]
		end

		it "returns number of rows updated" do
			adaptor.update(:person, :first_name => 'Jim').must_equal 3
		end
	end

	describe "#delete" do
		it "deletes all records" do
			adaptor.delete(:person)
			DB[:person].all.length.must_equal 0
		end

		it "deletes records conditionally" do
			adaptor.delete(:person, :first_name => "Tony")
			DB[:person].map {|person| person[:first_name] }.must_equal ["Peter", "Bruce"]
		end

		it "returns number of rows deleted" do
			adaptor.delete(:person, :first_name => "Tony").must_equal 1
		end
	end

	describe "#query" do
		it "returns selected records" do
			query_object = stub(:query => "SELECT * FROM person LIMIT 2")
			result = adaptor.query(query_object)
			result.map {|person| person[:first_name] }.must_equal ["Peter", "Tony"]
		end
	end

	describe "#transaction" do
		it "yields transaction to sequel" do
			adaptor.transaction do
				adaptor.update(:person, :first_name => 'Jim')
				adaptor.delete(:person, :last_name => "Stark")
			end
		end
	end
end