require 'spec_helper'
require 'storing/store'

describe Storing::Store do
	let(:table_name) { 'subscription' }
	let(:primary_key) { :subscription_id }
	let(:adaptor){ stub }
	let(:store) { Storing::Store.new(adaptor) }

	before do
		Storing::Store.set_primary_key primary_key
		Storing::Store.set_table_name table_name
	end

	after do
		Storing::Store.set_primary_key nil
		Storing::Store.set_table_name nil
	end

	describe "::set_primary_key" do
		it "sets primary_key" do
			Storing::Store.set_primary_key :user_id
			Storing::Store.primary_key.must_equal :user_id
		end
	end

	describe "::primary_key" do
		describe "when primary_key is assigned" do
			it "returns primary_key" do
				Storing::Store.set_primary_key :user_id
				Storing::Store.primary_key.must_equal :user_id
			end
		end

		describe "when primary_key is unassigned" do
			it "raises error" do
				Storing::Store.set_primary_key nil
				proc {Storing::Store.primary_key}.must_raise Storing::MissingPrimaryKey
			end
		end
	end

	describe "::set_table_name" do
		it "sets primary_key" do
			Storing::Store.set_table_name :users
			Storing::Store.table_name.must_equal :users
		end
	end

	describe "::table_name" do
		describe "when table_name is assigned" do
			it "returns table_name" do
				Storing::Store.set_table_name :user_id
				Storing::Store.table_name.must_equal :user_id
			end
		end

		describe "when table_name is unassigned" do
			it "raises error" do
				Storing::Store.set_table_name nil
				proc {Storing::Store.table_name}.must_raise Storing::MissingTableName
			end
		end
	end

	describe "#query" do
		it "delegates to adaptor" do
			arguments = stub
			adaptor.expects(:query).with(arguments)
			store.query(arguments)
		end
	end

	describe "#select" do
		it "delegates to adaptor with table name" do
			conditions = stub
			adaptor.expects(:select).with(table_name, conditions)
			store.select(conditions)
		end
	end

	describe "#insert" do
		it "delegates to adaptor with table name" do
			entity_hash = stub
			adaptor.expects(:insert).with(table_name, entity_hash)
			store.insert(entity_hash)
		end
	end

	describe "#update" do
		it "delegates to adaptor with table name" do
			entity_hash = stub
			params = stub
			primary_key_hash = stub
			entity = stub(:params => params, :primary_key_hash => primary_key_hash)
			store.expects(:new_entity).with(entity_hash).returns(entity)
			adaptor.expects(:update).with(table_name, params, primary_key_hash)
			store.update(entity_hash)
		end
	end

	describe "#delete" do
		it "delegates to adaptor with table name" do
			entity_hash = stub
			primary_key_hash = stub
			entity = stub(:primary_key_hash => primary_key_hash)
			store.expects(:new_entity).with(entity_hash).returns(entity)
			adaptor.expects(:delete).with(table_name, primary_key_hash)
			store.delete(entity_hash)
		end
	end

	describe "#transaction" do
		it "delegates to adaptor" do
			adaptor.expects(:transaction)
			store.transaction
		end
	end

	describe "#table_name" do
		it "returns class table_name" do
			Storing::Store.set_table_name 'my new table name'
			store.table_name.must_equal 'my new table name'
		end
	end

	describe "#primary_key" do
		it "returns class primary_key" do
			Storing::Store.set_primary_key 'my new primary key'
			store.primary_key.must_equal 'my new primary key'
		end
	end
end

describe Storing::Store::Entity do
	let(:data) {{:user_id => 123, :user_type => 'client', :age => 23, :first_name => 'John', :last_name => 'Doe'}}
	let(:primary_key_column){[:user_id, :user_type]}
	let(:entity_data) {Storing::Store::Entity.new(primary_key_column, data)}

	describe "#primary_key_hash" do
		describe "when primary_key is assigned" do
			it "returns primary key in hash format" do
				entity_data.primary_key_hash.must_equal({:user_id => 123, :user_type => 'client'})
			end
		end

		describe "when primary key key is missing from data" do
			let(:data) {{:user_id => 123, :age => 23, :first_name => 'John', :last_name => 'Doe'}}
			
			it "raises error" do
				proc{ entity_data.primary_key_hash }.must_raise(Storing::MissingPrimaryKey)
			end
		end
	end

	describe "#params" do
		it "returns data with primary key values removed" do
			entity_data.params.must_equal(:age => 23, :first_name => 'John', :last_name => 'Doe')
		end
	end
end